from django.db import models
from uuid import uuid4
from django.contrib.auth.models import AbstractUser as DjangoAbstractUser
from django.utils import timezone


# Create your models here.
class AbstractUser(DjangoAbstractUser):
    student_id = models.CharField(max_length=10, default="", null=True, blank=True, verbose_name="学籍番号")
    entry_year = models.IntegerField(null=True, blank=True, verbose_name="入学年度")
    created_at = models.DateTimeField(default=timezone.localtime, verbose_name="作成日")
    is_student = models.BooleanField(default=False, verbose_name="学生か")
    is_toyo_member = models.BooleanField(default=False, verbose_name="東洋大学内者か")
    is_iniad_member = models.BooleanField(default=False, verbose_name="INIAD学内者か")
    _grade_gap = models.IntegerField(default=0, verbose_name="ストレートとの学年の差")

    class Meta:
        ordering = ["-created_at"]
        abstract = True

    @property
    def grade(self):
        if self.is_student:
            today = timezone.localdate()
            return today.year - self.entry_year + (1 if today.month > 3 else 2) + self._grade_gap
        else:
            return None

    @grade.setter
    def grade(self, grade):
        if self.grade != grade:
            today = timezone.localdate()
            self._grade_gap = grade - (today.year - self.entry_year + (1 if today.month > 3 else 2))


class UUIDAbstractUser(AbstractUser):
    uuid = models.UUIDField(primary_key=True, editable=False, default=uuid4, verbose_name="UUID")

    class Meta:
        ordering = ["-created_at"]
        abstract = True



from django_toyo_auth.providers.toyo.provider import ToyoProvider, ToyoMail, ToyoAccount


class INIADMail(ToyoMail):
    domain = r"iniad\.org"

    def get_entry_class(self): # n期
        if self.get_is_student():
            return self.get_entry_year() - 2016
        else:
            return None


class INIADAccount(ToyoAccount):
    toyo_mail_class = INIADMail

    def get_entry_class(self):
        toyo_mail = self.toyo_mail_class(self.account.extra_data.get("email"))
        return toyo_mail.get_entry_class()


class INIADProvider(ToyoProvider):
    id = 'iniad'
    name = "INIAD"
    account_class = INIADAccount
    toyo_mail_class = INIADMail

    def get_default_scope(self):
        return [
            'profile',
            'email',
        ]

    def get_auth_params(self, request, action):
        ret = super(INIADProvider, self).get_auth_params(request, action)
        if not ret["access_type"]:
            ret["access_type"] = "online"
        ret["hd"] = "iniad.org"
        return ret

    def extract_extra_data(self, data):
        ret = super(INIADProvider, self).extract_extra_data(data)
        toyo_mail = self.toyo_mail_class(data.get("email"))
        ret.update({
            "is_iniad_member": True,
            "entry_class": toyo_mail.get_entry_class(),
        })
        return ret


provider_classes = [INIADProvider]



__copyright__    = 'Copyright (C) 2021 ayame.space'
__version__      = '0.7.0'
__license__      = 'MIT'
__author__       = 'ayame.space'
__author_email__ = 'ayame.space@gmail.com'
__url__          = 'http://github.com/ayame-q/django-toyo-auth'

import django

if django.VERSION < (3, 2):
    default_app_config = "django_toyo_auth.apps.AccountsConfig"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_toyo_auth',
 'django_toyo_auth.providers',
 'django_toyo_auth.providers.iniad',
 'django_toyo_auth.providers.toyo']

package_data = \
{'': ['*']}

install_requires = \
['Django', 'django-allauth']

setup_kwargs = {
    'name': 'django-toyo-auth',
    'version': '0.7.0',
    'description': 'It offers providers of Toyo University and INIAD Accounts for django-allauth',
    'long_description': '# Django Toyo Auth\n\n"Django Toyo Auth" offers providers of Toyo University Accounts(@toyo.jp) and INIAD Accounts(@iniad.org) for [django-allauth](https://django-allauth.readthedocs.io/en/latest/index.html)\n\n## Installation\n\n### Install Package\n\n```bash\npip install django-toyo-auth\n```\n\n### settings.py\n\n```python\nINSTALLED_APPS = [\n    ...\n    \'django.contrib.auth\',\n    \'django.contrib.messages\',\n    \'django.contrib.sites\',\n    \'allauth\',\n    \'allauth.account\',\n    \'allauth.socialaccount\',\n    \'django_toyo_auth\',\n    \'django_toyo_auth.providers.iniad\', # INIAD Account\n    \'django_toyo_auth.providers.toyo\', # Toyo Account\n    ...\n]\n\nSITE_ID = 1\n\n# Provider specific settings\nSOCIALACCOUNT_PROVIDERS = {\n    \'iniad\': { # for INIAD Account\n        \'SCOPE\': [\n            \'profile\',\n            \'email\',\n        ],\n        \'AUTH_PARAMS\': {\n            \'access_type\': \'online\',\n        },\n    },\n    \'toyo\': { # for Toyo Account\n        \'SCOPE\': [\n            \'profile\',\n            \'email\',\n        ],\n        \'AUTH_PARAMS\': {\n            \'access_type\': \'online\',\n        },\n    },\n}\n```\n\n### urls.py\n\n```python\nurlpatterns = [\n    ...\n    path(\'accounts/\', include(\'allauth.urls\')),\n    ...\n]\n```\n\n## Classes\n\n### django_toyo_auth.models.AbstractUser\n\nUser class with student_id, entry_year, is_student, is_toyo_member, is_iniad_member\n\n#### Attributes\n\n- student_id\n- entry_year\n- is_student\n- is_toyo_member\n- is_iniad_member\n- grade\n\n### django_toyo_auth.models.UUIDAbstractUser\n\nInherits all attributes and methods from [AbstractUser](django_toyo_auth.models.AbstractUser),\nbut also primary_key is UUID\n\n#### Attributes\n\n- uuid\n\n## Details\n\nIt offers only providers and custom models for django-allauth.\nPlease see [django-allauth documents](https://django-allauth.readthedocs.io/en/latest/index.html) for detail\n\n## Requirements\n\n- [Django](https://docs.djangoproject.com/)\n- [django-allauth](https://django-allauth.readthedocs.io/en/latest/index.html)\n\n## License\n\nMIT\n',
    'author': 'ayame.space',
    'author_email': 'ayame.space@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'http://github.com/ayame-q/django-toyo-auth',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
